from django.contrib import admin

from .models import LogEntry


@admin.register(LogEntry)
class LogEntryAdmin(admin.ModelAdmin):
    list_display = ('timestamp', 'level', 'message')
    list_filter = ('level', 'timestamp')
    search_fields = ('message',)
    date_hierarchy = 'timestamp'
    ordering = ('-timestamp',)


from django.db import models

from django.utils import timezone
import json


class LogLevel(models.TextChoices):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class LogEntry(models.Model):
    timestamp = models.DateTimeField(default=timezone.now, db_index=True)  # index for faster queries
    level = models.CharField(max_length=10, choices=LogLevel.choices)
    message = models.TextField()
    module = models.CharField(max_length=255, null=True, blank=True)
    function = models.CharField(max_length=255, null=True, blank=True)
    line_number = models.IntegerField(null=True, blank=True)
    user_id = models.IntegerField(null=True, blank=True)
    request_id = models.CharField(max_length=255, null=True, blank=True)
    extra_data = models.JSONField(default=dict, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)


    class Meta:
        ordering = ['-timestamp']
        indexes = [  # index for faster queries
            models.Index(fields=['timestamp']),
            models.Index(fields=['level']),
            models.Index(fields=['module']),
            models.Index(fields=['function']),
            models.Index(fields=['line_number']),
            models.Index(fields=['user_id']),
            models.Index(fields=['request_id']),
        ]
        verbose_name = 'Log Entry'
        verbose_name_plural = 'Log Entries'

    def __str__(self):
        """Return a string representation of the log entry."""
        return f"[{self.level}] {self.timestamp.strftime('%Y-%m-%d %H:%M:%S')} - {self.message[:100]}"
  


from django.apps import AppConfig


class LogqConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'logq'
    verbose_name = 'LogQ'

    def ready(self):
        """Initialize the async logger when the app is ready."""
        from .async_logger import get_async_logger
        get_async_logger()


# Generated by Django 5.2.4 on 2025-07-15 20:43

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LogEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('level', models.CharField(choices=[('DEBUG', 'Debug'), ('INFO', 'Info'), ('WARNING', 'Warning'), ('ERROR', 'Error'), ('CRITICAL', 'Critical')], max_length=10)),
                ('message', models.TextField()),
                ('module', models.CharField(max_length=255)),
                ('function', models.CharField(max_length=255)),
                ('line_number', models.IntegerField()),
                ('user_id', models.IntegerField(blank=True, null=True)),
                ('request_id', models.CharField(blank=True, max_length=255, null=True)),
                ('extra_data', models.JSONField(blank=True, default=dict)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Log Entry',
                'verbose_name_plural': 'Log Entries',
                'ordering': ['-timestamp'],
                'indexes': [models.Index(fields=['timestamp'], name='logq_logent_timesta_9e2e5a_idx'), models.Index(fields=['level'], name='logq_logent_level_fa122c_idx'), models.Index(fields=['module'], name='logq_logent_module_f632bb_idx'), models.Index(fields=['function'], name='logq_logent_functio_433e61_idx'), models.Index(fields=['line_number'], name='logq_logent_line_nu_bd9370_idx'), models.Index(fields=['user_id'], name='logq_logent_user_id_651abc_idx'), models.Index(fields=['request_id'], name='logq_logent_request_eda96e_idx')],
            },
        ),
    ]



from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods
from django.utils.decorators import method_decorator
from django.views import View
import json
from .async_logger import get_async_logger
from .models import LogEntry, LogLevel


@csrf_exempt
@require_http_methods(["POST"])
def log_endpoint(request):
    """Simple API endpoint for external logging."""
    try:
        data = json.loads(request.body)
        level = data.get('level', 'INFO')
        message = data.get('message', '')
        
        if not message:
            return JsonResponse({'error': 'Message is required'}, status=400)
        
        if level not in [choice[0] for choice in LogLevel.choices]:
            return JsonResponse({'error': 'Invalid log level'}, status=400)
        
        logger = get_async_logger()
        logger.log(level, message, extra_data=data.get('extra_data', {}))
        
        return JsonResponse({'status': 'success'})
    
    except json.JSONDecodeError:
        return JsonResponse({'error': 'Invalid JSON'}, status=400)
    except Exception as e:
        return JsonResponse({'error': str(e)}, status=500)


@method_decorator(csrf_exempt, name='dispatch')
class LogAPIView(View):
    """Class-based view for more advanced logging operations."""
    
    def post(self, request):
        """Handle POST requests for logging."""
        try:
            data = json.loads(request.body)
            level = data.get('level', 'INFO')
            message = data.get('message', '')
            
            if not message:
                return JsonResponse({'error': 'Message is required'}, status=400)
            
            logger = get_async_logger()
            logger.log(
                level, 
                message, 
                user_id=data.get('user_id'),
                request_id=data.get('request_id'),
                extra_data=data.get('extra_data', {})
            )
            
            return JsonResponse({'status': 'success'})
        
        except json.JSONDecodeError:
            return JsonResponse({'error': 'Invalid JSON'}, status=400)
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)
    
    def get(self, request):
        """Handle GET requests for retrieving recent logs."""
        try:
            limit = int(request.GET.get('limit', 100))
            level = request.GET.get('level')
            module = request.GET.get('module')
            
            query = LogEntry.objects.all()
            
            if level:
                query = query.filter(level=level)
            if module:
                query = query.filter(module__icontains=module)
            
            logs = query.order_by('-timestamp')[:limit]
            
            log_data = []
            for log in logs:
                log_data.append({
                    'id': log.id,
                    'timestamp': log.timestamp.isoformat(),
                    'level': log.level,
                    'message': log.message,
                    'module': log.module,
                    'function': log.function,
                    'line_number': log.line_number,
                    'user_id': log.user_id,
                    'request_id': log.request_id,
                    'extra_data': log.extra_data,
                })
            
            return JsonResponse({'logs': log_data})
        
        except ValueError:
            return JsonResponse({'error': 'Invalid limit parameter'}, status=400)
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)


from django.test import TransactionTestCase, override_settings
from django.urls import reverse
from django.contrib.auth.models import User
from django.utils import timezone
from django.db import connection
import json
import time
import threading
from .models import LogEntry, LogLevel
from .async_logger import AsyncLogger, get_async_logger, stop_async_logger


class AsyncLoggerTestCase(TransactionTestCase):
    def setUp(self):
        super().setUp()
        # Stop the global logger to avoid interference
        stop_async_logger()
        
        # Clear all existing logs using raw SQL to ensure complete cleanup
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        
        # Create a fresh logger instance for testing
        self.logger = AsyncLogger(max_queue_size=100, flush_interval=0.1)
        self.logger.start()
        time.sleep(0.2)  # Wait for thread to start
    
    def tearDown(self):
        self.logger.stop()
        time.sleep(0.2)  # Wait for thread to stop
        
        # Clear logs after test using raw SQL
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        
        super().tearDown()
    
    def test_basic_logging(self):
        """Test basic logging functionality."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        self.logger.info("Test message")
        time.sleep(0.5)  # Wait longer for flush
        
        # Verify we have exactly one log entry
        self.assertEqual(LogEntry.objects.count(), 1)
        
        log_entry = LogEntry.objects.first()
        self.assertEqual(log_entry.level, LogLevel.INFO)
        self.assertEqual(log_entry.message, "Test message")
    
    def test_all_log_levels(self):
        """Test all log levels."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        levels = [LogLevel.DEBUG, LogLevel.INFO, LogLevel.WARNING, LogLevel.ERROR, LogLevel.CRITICAL]
        
        for level in levels:
            self.logger.log(level, f"Test {level}")
        
        time.sleep(0.5)  # Wait longer for flush
        
        entries = LogEntry.objects.all()
        self.assertEqual(entries.count(), len(levels))
        
        for entry in entries:
            self.assertIn(entry.level, levels)
    
    def test_extra_data(self):
        """Test logging with extra data."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        extra_data = {'user_id': 123, 'action': 'test'}
        self.logger.info("Test with extra data", extra_data=extra_data)
        time.sleep(0.5)
        
        # Verify we have exactly one log entry
        self.assertEqual(LogEntry.objects.count(), 1)
        
        entry = LogEntry.objects.first()
        self.assertEqual(entry.extra_data, extra_data)
    
    def test_queue_full_handling(self):
        """Test behavior when queue is full."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        # Fill the queue
        for i in range(150):  # More than max_queue_size
            self.logger.info(f"Message {i}")
        
        time.sleep(0.5)
        
        # Should have some entries but not all due to queue being full
        entries = LogEntry.objects.count()
        self.assertGreater(entries, 0)
        self.assertLessEqual(entries, 100)  # max_queue_size


class LogEntryModelTestCase(TransactionTestCase):
    def setUp(self):
        super().setUp()
        # Clear all existing logs
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
    
    def tearDown(self):
        # Clear logs after test
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        super().tearDown()
    
    def test_log_entry_creation(self):
        """Test LogEntry model creation."""
        entry = LogEntry.objects.create(
            level=LogLevel.INFO,
            message="Test message",
            module="test_module",
            function="test_function",
            line_number=42,
            user_id=123,
            request_id="test-request-id",
            extra_data={'key': 'value'}
        )
        
        self.assertEqual(entry.level, LogLevel.INFO)
        self.assertEqual(entry.message, "Test message")
        self.assertEqual(entry.module, "test_module")
        self.assertEqual(entry.function, "test_function")
        self.assertEqual(entry.line_number, 42)
        self.assertEqual(entry.user_id, 123)
        self.assertEqual(entry.request_id, "test-request-id")
        self.assertEqual(entry.extra_data, {'key': 'value'})
    
    def test_log_entry_str_representation(self):
        """Test string representation of LogEntry."""
        entry = LogEntry.objects.create(
            level=LogLevel.ERROR,
            message="This is a very long message that should be truncated in the string representation",
            timestamp=timezone.now()
        )
        
        str_repr = str(entry)
        self.assertIn("[ERROR]", str_repr)
        self.assertIn("This is a very long message that should be truncated", str_repr[:100])


class LoggingAPITestCase(TransactionTestCase):
    def setUp(self):
        super().setUp()
        self.user = User.objects.create_user(username='testuser', password='testpass')
        # Stop the global logger to avoid interference
        stop_async_logger()
        # Clear all existing logs
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
    
    def tearDown(self):
        # Clear logs after test
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        super().tearDown()
    
    def test_log_endpoint(self):
        """Test the log API endpoint."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        data = {
            'level': 'INFO',
            'message': 'Test API log',
            'extra_data': {'source': 'api'}
        }
        
        response = self.client.post(
            reverse('logq:log_endpoint'),
            data=json.dumps(data),
            content_type='application/json'
        )
        
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()['status'], 'success')
        
        # Wait for async processing
        time.sleep(0.5)
        
        # Verify we have exactly one log entry
        self.assertEqual(LogEntry.objects.count(), 1)
        
        entry = LogEntry.objects.first()
        self.assertEqual(entry.message, 'Test API log')
        self.assertEqual(entry.extra_data, {'source': 'api'})
    
    def test_log_api_view(self):
        """Test the class-based log API view."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        data = {
            'level': 'WARNING',
            'message': 'Test warning',
            'user_id': self.user.id,
            'request_id': 'test-123'
        }
        
        response = self.client.post(
            reverse('logq:log_api'),
            data=json.dumps(data),
            content_type='application/json'
        )
        
        self.assertEqual(response.status_code, 200)
        
        time.sleep(0.5)
        
        # Verify we have exactly one log entry
        self.assertEqual(LogEntry.objects.count(), 1)
        
        entry = LogEntry.objects.first()
        self.assertEqual(entry.level, LogLevel.WARNING)
        self.assertEqual(entry.user_id, self.user.id)
        self.assertEqual(entry.request_id, 'test-123')
    
    def test_get_logs_api(self):
        """Test retrieving logs via API."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        # Create some test logs directly
        LogEntry.objects.create(level=LogLevel.INFO, message="Test 1")
        LogEntry.objects.create(level=LogLevel.ERROR, message="Test 2")
        LogEntry.objects.create(level=LogLevel.DEBUG, message="Test 3")
        
        # Verify we have exactly 3 logs
        self.assertEqual(LogEntry.objects.count(), 3)
        
        response = self.client.get(reverse('logq:log_api'))
        self.assertEqual(response.status_code, 200)
        
        data = response.json()
        self.assertEqual(len(data['logs']), 3)
        self.assertEqual(data['logs'][0]['message'], "Test 1")
    
    def test_invalid_log_level(self):
        """Test API with invalid log level."""
        data = {
            'level': 'INVALID',
            'message': 'Test message'
        }
        
        response = self.client.post(
            reverse('logq:log_endpoint'),
            data=json.dumps(data),
            content_type='application/json'
        )
        
        self.assertEqual(response.status_code, 400)
        self.assertIn('Invalid log level', response.json()['error'])


@override_settings(ASYNC_LOGGING_CONFIG={'MAX_QUEUE_SIZE': 500, 'FLUSH_INTERVAL': 0.5})
class ConfigurationTestCase(TransactionTestCase):
    def setUp(self):
        super().setUp()
        # Clear all existing logs
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
    
    def tearDown(self):
        # Clear logs after test
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        super().tearDown()
    
    def test_custom_configuration(self):
        """Test that custom configuration is respected."""
        logger = AsyncLogger()
        self.assertEqual(logger.queue.maxsize, 500)
        self.assertEqual(logger.flush_interval, 0.5)


class MiddlewareTestCase(TransactionTestCase):
    def setUp(self):
        super().setUp()
        # Stop the global logger to avoid interference
        stop_async_logger()
        # Clear all existing logs
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
    
    def tearDown(self):
        # Clear logs after test
        with connection.cursor() as cursor:
            cursor.execute("DELETE FROM logq_logentry")
        super().tearDown()
    
    def test_middleware_request_logging(self):
        """Test that middleware logs requests."""
        # Verify we start with no logs
        self.assertEqual(LogEntry.objects.count(), 0)
        
        response = self.client.get('/admin/')
        
        time.sleep(0.5)
        
        entries = LogEntry.objects.all()
        self.assertGreater(entries.count(), 0)
        
        # Should have request start and completion logs
        start_logs = entries.filter(message__contains="Request started")
        complete_logs = entries.filter(message__contains="Request completed")
        
        self.assertGreater(start_logs.count(), 0)
        self.assertGreater(complete_logs.count(), 0)



# Generated by Django 5.2.4 on 2025-07-15 21:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('logq', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='logentry',
            name='function',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='line_number',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='module',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from .async_logger import get_async_logger
from functools import wraps
import time


def log_function_call(func=None, *, level='INFO'):
    """
    Decorator to automatically log function calls.
    
    Usage:
        @log_function_call
        def my_function():
            pass
        
        @log_function_call(level='DEBUG')
        def my_debug_function():
            pass
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            logger = get_async_logger()
            
            # Log function entry
            logger.log(
                level,
                f"Entering function: {func.__name__}",
                extra_data={
                    'args_count': len(args),
                    'kwargs_keys': list(kwargs.keys()),
                    'module': func.__module__,
                }
            )
            
            start_time = time.time()
            try:
                result = func(*args, **kwargs)
                execution_time = time.time() - start_time
                
                # Log successful completion
                logger.log(
                    level,
                    f"Function completed: {func.__name__} (took {execution_time:.3f}s)",
                    extra_data={'execution_time': execution_time}
                )
                
                return result
            
            except Exception as e:
                execution_time = time.time() - start_time
                
                # Log exception
                logger.exception(
                    f"Function failed: {func.__name__} (took {execution_time:.3f}s)",
                    exc_info=str(e),
                    extra_data={'execution_time': execution_time}
                )
                raise
        
        return wrapper
    
    if func is None:
        return decorator
    return decorator(func)


def log_performance(threshold_seconds=1.0):
    """
    Decorator to log slow function calls.
    
    Usage:
        @log_performance(threshold_seconds=0.5)
        def my_slow_function():
            pass
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            start_time = time.time()
            result = func(*args, **kwargs)
            execution_time = time.time() - start_time
            
            if execution_time > threshold_seconds:
                logger = get_async_logger()
                logger.warning(
                    f"Slow function detected: {func.__name__} took {execution_time:.3f}s",
                    extra_data={
                        'execution_time': execution_time,
                        'threshold': threshold_seconds,
                        'module': func.__module__,
                    }
                )
            
            return result
        return wrapper
    return decorator


class LogContext:
    """
    Context manager for logging operations with automatic timing.
    
    Usage:
        with LogContext("Processing data", level='INFO'):
            # do some work
            pass
    """
    
    def __init__(self, message, level='INFO', **kwargs):
        self.message = message
        self.level = level
        self.kwargs = kwargs
        self.logger = get_async_logger()
        self.start_time = None
    
    def __enter__(self):
        self.start_time = time.time()
        self.logger.log(
            self.level,
            f"Starting: {self.message}",
            **self.kwargs
        )
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        execution_time = time.time() - self.start_time
        
        if exc_type is None:
            self.logger.log(
                self.level,
                f"Completed: {self.message} (took {execution_time:.3f}s)",
                extra_data={'execution_time': execution_time},
                **self.kwargs
            )
        else:
            self.logger.exception(
                f"Failed: {self.message} (took {execution_time:.3f}s)",
                exc_info=str(exc_val),
                extra_data={'execution_time': execution_time},
                **self.kwargs
            )


from django.urls import path
from . import views

app_name = 'logq'

urlpatterns = [
    path('api/log/', views.log_endpoint, name='log_endpoint'),
    path('api/logs/', views.LogAPIView.as_view(), name='log_api'),
]



from django.core.management.base import BaseCommand
from django.utils import timezone
from datetime import timedelta
from django.conf import settings
from logq.models import LogEntry


class Command(BaseCommand):
    help = 'Clean old log entries from the database'
    
    def add_arguments(self, parser):
        parser.add_argument(
            '--days',
            type=int,
            default=None,
            help='Delete logs older than this many days (default: from ASYNC_LOGGING_CONFIG)'
        )
        parser.add_argument(
            '--level',
            type=str,
            choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
            help='Only delete logs of this level or lower'
        )
        parser.add_argument(
            '--dry-run',
            action='store_true',
            help='Show what would be deleted without actually deleting'
        )
    
    def handle(self, *args, **options):
        # Get default days from settings
        config = getattr(settings, 'ASYNC_LOGGING_CONFIG', {})
        default_days = config.get('AUTO_CLEANUP_DAYS', 30)
        
        days = options['days'] if options['days'] is not None else default_days
        level = options['level']
        dry_run = options['dry_run']
        
        # Calculate cutoff date
        cutoff_date = timezone.now() - timedelta(days=days)
        
        # Build query
        query = LogEntry.objects.filter(timestamp__lt=cutoff_date)
        
        if level:
            # Get level hierarchy
            level_order = ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']
            level_index = level_order.index(level)
            levels_to_delete = level_order[:level_index + 1]
            query = query.filter(level__in=levels_to_delete)
        
        count = query.count()
        
        if dry_run:
            self.stdout.write(
                self.style.WARNING(
                    f'Would delete {count} log entries older than {days} days'
                    + (f' with level {level} or lower' if level else '')
                )
            )
        else:
            deleted_count = query.delete()[0]
            self.stdout.write(
                self.style.SUCCESS(
                    f'Successfully deleted {deleted_count} log entries older than {days} days'
                    + (f' with level {level} or lower' if level else '')
                )
            )


import uuid
from django.utils.deprecation import MiddlewareMixin
from .async_logger import get_async_logger
from django.conf import settings

class AsyncLoggingMiddleware(MiddlewareMixin):
    """
    Middleware that automatically logs request information and adds request_id to the request.
    """
    
    def process_request(self, request):
        # Check if request path is in ignore paths
        ignore_paths = getattr(settings, 'ASYNC_LOGGING_CONFIG', {}).get('IGNORE_PATHS', [])
        if any(request.path.startswith(path) for path in ignore_paths):
            return None
        
        # Generate unique request ID
        request.request_id = str(uuid.uuid4())
        
        # Get user ID if authenticated
        user_id = None
        if hasattr(request, 'user') and request.user.is_authenticated:
            user_id = request.user.id
        
        # Log request start
        logger = get_async_logger()
        logger.info(
            f"Request started: {request.method} {request.path}",
            request_id=request.request_id,
            user_id=user_id,
            extra_data={
                'method': request.method,
                'path': request.path,
                'query_params': dict(request.GET),
                'user_agent': request.META.get('HTTP_USER_AGENT', ''),
                'ip_address': self._get_client_ip(request),
            }
        )
    
    def process_response(self, request, response):
        # Check if request path is in ignore paths
        ignore_paths = getattr(settings, 'ASYNC_LOGGING_CONFIG', {}).get('IGNORE_PATHS', [])
        if any(request.path.startswith(path) for path in ignore_paths):
            return response
        
        # Get user ID if authenticated
        user_id = None
        if hasattr(request, 'user') and request.user.is_authenticated:
            user_id = request.user.id
        
        # Log request completion
        logger = get_async_logger()
        logger.info(
            f"Request completed: {request.method} {request.path} - {response.status_code}",
            request_id=getattr(request, 'request_id', 'unknown'),
            user_id=user_id,
            extra_data={
                'status_code': response.status_code,
                'content_length': len(response.content) if hasattr(response, 'content') else 0,
            }
        )
        
        return response
    
    def process_exception(self, request, exception):
        # Check if request path is in ignore paths
        ignore_paths = getattr(settings, 'ASYNC_LOGGING_CONFIG', {}).get('IGNORE_PATHS', [])
        if any(request.path.startswith(path) for path in ignore_paths):
            return None
        
        # Get user ID if authenticated
        user_id = None
        if hasattr(request, 'user') and request.user.is_authenticated:
            user_id = request.user.id
        
        # Log exception
        logger = get_async_logger()
        logger.exception(
            f"Request exception: {request.method} {request.path}",
            exc_info=str(exception),
            request_id=getattr(request, 'request_id', 'unknown'),
            user_id=user_id,
            extra_data={
                'exception_type': type(exception).__name__,
                'exception_args': str(exception.args),
            }
        )
    
    def _get_client_ip(self, request):
        """Get the client's IP address."""
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip


import threading
import queue
import time
import logging
import traceback
import inspect
from typing import Optional, Dict, Any
from django.utils import timezone
from django.db import transaction
from django.conf import settings
from .models import LogEntry, LogLevel


class AsyncLogger:
    """
    Asynchronous logger that runs in a separate thread to avoid blocking the main application.
    """
    
    def __init__(self, max_queue_size: int = None, flush_interval: float = None):
        # Get configuration from settings
        config = getattr(settings, 'ASYNC_LOGGING_CONFIG', {})
        self.max_queue_size = max_queue_size or config.get('MAX_QUEUE_SIZE', 1000)
        self.flush_interval = flush_interval or config.get('FLUSH_INTERVAL', 1.0)
        
        self.queue = queue.Queue(maxsize=self.max_queue_size)
        self.running = False
        self.thread = None
        self._lock = threading.Lock()
        
    def start(self):
        """Start the logging thread."""
        with self._lock:
            if not self.running:
                self.running = True
                self.thread = threading.Thread(target=self._worker, daemon=True)
                self.thread.start()
    
    def stop(self):
        """Stop the logging thread."""
        with self._lock:
            if self.running:
                self.running = False
                if self.thread:
                    self.thread.join(timeout=5.0)
    
    def _worker(self):
        """Worker thread that processes log entries from the queue."""
        batch = []
        last_flush = time.time()
        
        while self.running:
            try:
                # Try to get a log entry with timeout
                try:
                    entry = self.queue.get(timeout=0.1)
                    batch.append(entry)
                except queue.Empty:
                    pass
                
                # Flush batch if it's time or batch is getting large
                current_time = time.time()
                if (current_time - last_flush >= self.flush_interval or 
                    len(batch) >= 50):
                    if batch:
                        self._flush_batch(batch)
                        batch = []
                        last_flush = current_time
                        
            except Exception as e:
                # Log the error to prevent infinite loops
                print(f"Error in async logger worker: {e}")
                time.sleep(1)
        
        # Flush remaining entries
        if batch:
            self._flush_batch(batch)
    
    def _flush_batch(self, batch):
        """Flush a batch of log entries to the database."""
        try:
            with transaction.atomic():
                LogEntry.objects.bulk_create(batch, ignore_conflicts=True)
        except Exception as e:
            print(f"Error flushing log batch: {e}")
    
    def log(self, level: str, message: str, **kwargs):
        """Add a log entry to the queue."""
        if not self.running:
            return
        
        # Get caller information
        frame = inspect.currentframe().f_back
        module = frame.f_globals.get('__name__', 'unknown')
        function = frame.f_code.co_name
        line_number = frame.f_lineno
        
        # Create log entry
        entry = LogEntry(
            level=level,
            message=message,
            module=module,
            function=function,
            line_number=line_number,
            user_id=kwargs.get('user_id'),
            request_id=kwargs.get('request_id'),
            extra_data=kwargs.get('extra_data', {})
        )
        
        try:
            self.queue.put_nowait(entry)
        except queue.Full:
            # If queue is full, log to console as fallback
            print(f"Log queue full, dropping entry: [{level}] {message}")
    
    def debug(self, message: str, **kwargs):
        self.log(LogLevel.DEBUG, message, **kwargs)
    
    def info(self, message: str, **kwargs):
        self.log(LogLevel.INFO, message, **kwargs)
    
    def warning(self, message: str, **kwargs):
        self.log(LogLevel.WARNING, message, **kwargs)
    
    def error(self, message: str, **kwargs):
        self.log(LogLevel.ERROR, message, **kwargs)
    
    def critical(self, message: str, **kwargs):
        self.log(LogLevel.CRITICAL, message, **kwargs)
    
    def exception(self, message: str, exc_info=None, **kwargs):
        """Log an exception with traceback."""
        if exc_info is None:
            exc_info = traceback.format_exc()
        
        extra_data = kwargs.get('extra_data', {})
        extra_data['traceback'] = exc_info
        
        self.log(LogLevel.ERROR, message, extra_data=extra_data, **kwargs)


# Global logger instance
_async_logger = None


def get_async_logger() -> AsyncLogger:
    """Get the global async logger instance."""
    global _async_logger
    if _async_logger is None:
        _async_logger = AsyncLogger()
        _async_logger.start()
    return _async_logger


def stop_async_logger():
    """Stop the global async logger."""
    global _async_logger
    if _async_logger:
        _async_logger.stop()
        _async_logger = None

